// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *			Basic Dialog class for XVT
 *
 * The present file declares XVTModalDialog and XVTDialog classes for
 * handling modal and modeless dialogs under XVT.
 *
 * Uses the Modal dialog - it is assumed the user is asked a question and he's
 * expected to give an answer (by entering some info or clicking an appropriate
 * button or selecting smth from the list, etc). We assume that there is always
 * a "Cancel" button, meaning that the answer is NO.
 * When the dialog is created, it shows the window and lets the user to make up
 * his mind and select smth. The window is closed when the selection is made.
 * So, after the dialog has been created, the entire show is over, and the
 * dialog object keeps the result of the selection.
 * No other events are handled, redefine the event handler if necessary.
 * Note this is an abstract class - the user has to redefine its own handler
 * of events occuring on a control item, plus the user shouldn't forget
 * to call commence() function it his constructor
 *
 * Modeless Dialog is almost analogous to a Window with controls, with the
 * only difference being that the modeless dialog does not get Update/Menu
 * selection/mouse movement/button pressing events. Unfortunately, the
 * modeless dialog goes the way of a modal dialog in one respect: it
 * gets the CREATE event right away after the create_res_dialog(), without
 * even waiting for the window to show up (and without even waiting for
 * the entire chain of constructors to finish). It means that the constructor
 * for the dialog does not create the dialog at all. To do that, one
 * needs to call 'commence()', which really constructs the dialog and starts
 * the event processing. Still, to keep getting events, one shouldn't forget
 * to call 'serve_to_death()' function somewhere.
 *
 * Note, since constructors (for both modal and modeless dialogs) don't
 * actually create anything, they're made protected. Just to avoid
 * confusion: otherwise, one way expect something would show up once he
 * created an object. Sorry, not until commence() is called, which should
 * be done somewhere in the constructor of a derived class (at the very end
 * of inheritance chain).
 *
 * $Id: Dialog.h,v 2.0 1994/07/06 18:55:04 oleg Exp oleg $
 *
 *************************************************************************
 */

#ifndef __GNUC__
#pragma once
#else
#pragma interface
#endif
#include "Window.h"

class XVTModalDialog : public XVTBasicWindow
{
  short  dialog_id;

  void draw(void) {}		// It's never called in dialogs anyway

protected:
  void commence(void);		// This function actually creates the dialog

  XVTModalDialog(const short resource_id);
  
	
public:
				// Create a dialog defined in the resource file
  ~XVTModalDialog(void) {}
};


class XVTDialog : public XVTBasicWindow
{
  short  dialog_id;

  void draw(void) {}		// It's never called in dialogs anyway

protected:
  XVTDialog(const short resource_id);
  void commence(void);		// This function actually creates the dialog

public:

  ~XVTDialog(void)		{}
};

