// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *		     Color Lookup Manipulation Classes
 *
 * The classes and procedures defined in here let one manipulate colorcells
 * of the gray color-lookup table used in displaying grayscale images.
 * Namely, one can assign non-gray color to one or several clut entries
 * thus pseudo-colorizing the image. Class CLUTStrip is defined to provide
 * a front-end user interface to the gray CLUT, that is to display the
 * CLUT and let the user tinker about it.
 *
 * $Id: clut_manip.h,v 1.1 1994/10/03 13:54:29 oleg Exp oleg $
 *
 *************************************************************************
 */

#ifndef __GNUC__
#pragma once
#else
#pragma interface
#endif

#include "image_win.h"

class PseudocolorCLUTRange: CustomCLUT
{
  int cmap_range;			// Number of cmap entries affected
  struct GrayCLUTEntry
  {
    GRAYSHADE index;
    COLOR gray_contents;
  }
  *cmap_saved_range;			// Saved gray shades of the affected
					// cmap entries

public:					// Pseudo-colorize pixels from lo
					// up to hi
  PseudocolorCLUTRange(const GRAY lo, const GRAY hi, const XVTColor
		       pseudocolor);
  ~PseudocolorCLUTRange(void);		// This would unpseudocolor affected
					// pixels
};


class CLUTStrip : XVTImageWindow
{
  virtual void handler(EVENT event);	// A bit more specific handler
  const int pseudo_coloring_width;
  PseudocolorCLUTRange * pseudocolorized_range;
  XVTColor pseudo_color;

					// Handle the mouse click to replace
					// a section of CLUT with a pseudo-
					// color
  void handle_mouse_click(const PNT where);

public:
  CLUTStrip(const char * title="Gray Shades");
  ~CLUTStrip(void);
};
