// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *			Verify Image Display under XVT
 *
 * Check out (almost) all functionality defined in image_win.cc
 *
 * $Id: vimage_win.cc,v 1.3 1994/10/06 15:25:24 oleg Exp oleg $
 *
 *************************************************************************
 */

#include "myenv.h"
#include "SimpleProcess.h"
#include "clut_manip.h"
#include "rubber_rect.h"

#include "vimage_intf.h"
#include <builtin.h>

/*
 *------------------------------------------------------------------------
 *			Making some test images
 */

				// Make a diagonal ramp with a white
				// line across the diagonal
static IMAGE * make_diag_ramp(void)
{
  IMAGE& image = *(new IMAGE(256,256,8));
  
  register int i,j;
  for(i=0; i<image.q_ncols(); i++)
    for(j=0; i+j < image.q_nrows(); j++)
	image(j,i+j) = image(i+j,j) = i;
  for(i=0; i<image.q_nrows(); i++)
    image(i,image.q_nrows()-1-i) = 255;

  return &image;
}


				// Make a gradient fill (stripy image)
				// with the white square in the middle
static void make_fill_and_square(IMAGE& image)
{
  register int j;
  for(j=0; j<image.q_ncols(); j++)
    image.rectangle(rowcol(0,j),rowcol(image.q_nrows()-1,j)) = j % 256;

  image.square_of(min(image.q_ncols(),image.q_nrows())/2,
		  rowcol(image.q_nrows()/4,image.q_ncols()/4)) = 255;
}

/*
 *------------------------------------------------------------------------
 *			non-GUI Test modules
 */

static void test_color_conversions(void)
{
  message("\nTesting color conversions\n");
  struct RGB { int red, green, blue; } rgbs [] =
  { {0,0,0}, {255,255,255}, {1,129,255}, {255,1,129}, {129,255,1} };

  for(register int i=0; i<sizeof(rgbs)/sizeof(rgbs[0]); i++)
  {
    const RGB& rgb = rgbs[i];

    XVTColor color(rgb.red,rgb.green,rgb.blue);
    COLOR xvt_native_color = color;
    XVTColor color1(xvt_native_color);
    assert( xvt_native_color == (COLOR)color1 );

    char buffer[40];
    sprintf(buffer,"#%1x%1x%1x",rgb.red & 0xf, rgb.green & 0xf, 
	    rgb.blue & 0xf );
    message("\ttesting rgb string '%s'\n",buffer);
    XVTColor color2(buffer);
    COLOR xvt_native_color2 = color2;
    XVTColor color3(xvt_native_color2);
    assert( xvt_native_color2 == (COLOR)color3 );
    
    sprintf(buffer," #%02x%02x%02x ", rgb.red, rgb.green, rgb.blue);
    message("\ttesting rgb string '%s'\n",buffer);
    XVTColor color4(buffer);
    COLOR xvt_native_color4 = color4;
    assert( xvt_native_color4 == xvt_native_color );
    XVTColor color5(xvt_native_color4);
    assert( xvt_native_color4 == (COLOR)color5 );
  }
  message("\nDone\n");
}

/*
 *------------------------------------------------------------------------
 *			    GUI Test modules
 */

static void test_diag_ramp(void)
{
  IMAGE& test_image = *make_diag_ramp();
  XVTImageWindow image_w(IDS_IMAGE,test_image,"Diag Ramp",TASK_WIN);
  image_w.show();
//    image_w.write_pgm("/tmp/ab.pgm");
  assert( image_w == test_image );
  image_w.serve_to_death();
  delete &test_image;
}  

static void test_grad_fill_with_square(void)
{
  XVTImageWindow image_w(IDS_IMAGE,256,512,
			 "Gradient fill & white square",TASK_WIN);
  make_fill_and_square(image_w);
  image_w.refresh();
  image_w.redraw();
  image_w.show();
  assert( ((XVTRect)(XVTWindow&)image_w).q_width() == 256 );
  assert( ((XVTRect)(XVTWindow&)image_w).q_height() == 256 );
  image_w.serve_to_death();
}


static void test_pseudocoloring(void)
{
  IMAGE& test_image = *make_diag_ramp();
  XVTImageWindow image_w(IDS_IMAGE,test_image,"Diag Ramp",TASK_WIN);
  image_w.show();
  assert( image_w == test_image );
  CLUTStrip clut_strip;
  image_w.serve_to_death();
  delete &test_image;
}  



class SelectableWindow;
class MyRubberRectangle : public RubberRectangle
{
  SelectableWindow& dialog;
  BOOLEAN processed_changed_rect(const XVTRect& rect) const;
  public:
  MyRubberRectangle(SelectableWindow& _dialog, const XVTImageWindow& parent)
    : dialog(_dialog), RubberRectangle(parent) {}
  ~MyRubberRectangle(void) {}
};

class SelectableWindow : public XVTWindow
{
  const IMAGE& image;
  XVTImageWindow *imagewp;
  MyRubberRectangle * rubber_rectp;

  BOOLEAN handle_ctl(const short id, const CONTROL_INFO& ctl)
  {
    if( id == IDR_SHOW )
      (*rubber_rectp).open();
    if( id == IDR_HIDE )
      (*rubber_rectp).cancel();
    return TRUE;
  }
  void cancel(void) { (*rubber_rectp).cancel(); (*imagewp).cancel();
		      XVTWindow::cancel(); }
  BOOLEAN initialize(void)
  {
    imagewp = new XVTImageWindow(IDR_IMAGE,image,"title",q_window());
    rubber_rectp = new MyRubberRectangle(*this,*imagewp);
    (*imagewp).show();
    return TRUE;
  }
  void draw(void) {}		// Nothing to draw -this window is a dialog
				// actually
public:
  SelectableWindow(const IMAGE& _image, const char * title)
      : image(_image),
      imagewp(0),
      rubber_rectp(0) { create(IDR_RECT,TASK_WIN); }
  ~SelectableWindow(void) { delete imagewp; delete rubber_rectp; }
};

BOOLEAN MyRubberRectangle::processed_changed_rect(const XVTRect& rect) const
{ char buffer[100];
  sprintf(buffer,"(%d,%d) width %d height %d",rect.left, rect.top,
	  rect.q_width(), rect.q_height());
  dialog.set_title(IDR_RPOS,buffer);
  return TRUE;
}

static void test_rubber_rect(void)
{
  IMAGE test_image(256,512,8);
  make_fill_and_square(test_image);
  SelectableWindow image_w(test_image,"Gradiend fill & white square");
  image_w.show();
  image_w.serve_to_death();
}

/*
 *------------------------------------------------------------------------
 *		A process root module that the process runs
 */

SimpleProcess::SimpleProcess(void)
{
  config.menu_bar_ID = M_MENUBAR;
  config.about_box_ID = 0;
  config.appl_name = config.base_appl_name = "vimage_win";
  config.taskwin_title = "Verify Gray Image Display";
}

 
void SimpleProcess::start(void)
{
//	xvt_note("Working");

  test_color_conversions();

  test_diag_ramp();

  test_grad_fill_with_square();

  test_pseudocoloring();

  test_rubber_rect();

  xvt_note("Finished");
  xvt_terminate();
}
